#' flr_test
#'
#' @param data A flowering time data frame
#' @param form A vector of formulae representing the comparisons I wish to perform
#' @param group What groups I am comparing ("sowing_number", "genotype" or "mutant")
#' @param species Either "Medicago" (default) of "Arabidopsis"
#' @param diff_geno This boolean parameter is ony relevent when group is set to "mutant".
#' It allows multiple genotypes to be tested at once against one another.
#' (e.g. A segregating population vs wild-type). I generally have this turned off as a
#' safty precaution so unexpected things arn't accidentally grouped.
#' @description Takes a flowering time dataset and performs a Hotellings \eqn{T^2}
#' on flowering time data and reports results in reader friendly wide format.
#' Will perform multiple testing correction (FDR) when needed.
#'
#' @examples
#' forms <- c(Q758 ~ Q756, Q758 ~ Q757)
#' flr_test(Mt_flowering_time_data, forms)
#' @export
flr_test <- function(data, form, group = sowing_number,
                     species = "Medicago", diff_geno = FALSE){

  require(rlang)
  require(dplyr)
  require(tidyr)
  require(purrr)
  require(magrittr)
  require(ICSNP)
  require(broom)
  require(checkmate)

  # Make group a quosure
  group <- enquo(group)

  # Argument Checks
  assertDataFrame(data)
  assertNames(quo_name(group), subset.of =  c("sowing_number", "genotype", "mutant"))
  assert_logical(diff_geno)

  # Put formula in a list
  if(is_formula(form) == TRUE){
    form <- list(form)
  }

  assert_list(form)

  form_check <- map_lgl(form, ~ is_formula(.x) & length(.x) == 3)
  if(sum(form_check) != length(form_check)) {
    stop("form must be a formula or a list of formulas, each with exactly one
         element on each side.")
  }

  # Make group a quosure
  group <- enquo(group)

  # Table of contrasts
  test_tbl <- data_frame(LHS = map_chr(form, ~ .x[[2]] %>%
                                         toString()),
                         RHS =  map_chr(form, ~ .x[[3]] %>%
                                          toString())
  )

  # Check only one entry in LHS and RHS
  form_check <- c(test_tbl$LHS, test_tbl$RHS) %>%
    .nwords(pseudo = TRUE) %>%
    max()
  if(form_check != 1){
    stop("form must be a list of formulas, each with exactly one
         element on each side.")
  }

  # Structure data based on queries
  if(quo_name(group) == "genotype"){

    data_subs <- data %>%
      select(genotype, conditions) %>%
      distinct()

    test_tbl <- test_tbl %>%
      mutate(count=1:n()) %>%
      gather(side, genotype, -count) %>%
      left_join(data_subs, by = "genotype") %>%
      spread(side, genotype) %>%
      select(LHS, RHS, conditions, -count)

    test_tbl <- test_tbl %>%
      mutate(raw_data = pmap(list(.$LHS, .$RHS, .$conditions),
                             ~ filter(data, genotype == ..1 | genotype == ..2,
                                      conditions == ..3)),
             sowing_numbers = map(raw_data, ~ unique(.x$sowing_number)))

  }
  else if(quo_name(group) == "mutant"){

    data_subs <- data %>%
      select(genotype, conditions, mutant) %>%
      distinct()

    test_tbl <- test_tbl %>%
      mutate(count=1:n()) %>%
      gather(side, mutant, -count) %>%
      left_join(data_subs, by = "mutant")

    if(diff_geno == TRUE){
      message("Single Genotype restriction turned off - see diff_geno parameter")

      test_tbl <- test_tbl %>%
        select(-genotype) %>%
        spread(side, mutant) %>%
        select(LHS, RHS, conditions, -count) %>%
        mutate(raw_data = pmap(list(.$LHS, .$RHS, .$conditions),
                               ~ filter(data, mutant == ..1 | mutant == ..2,
                                        conditions == ..3)),
               sowing_numbers = map(raw_data, ~ unique(.x$sowing_number)))
    }
    else{
      if(test_tbl$genotype %>% unique() %>% length() > 1){stop("Multiple genotypes?? Do you want to change the diff_geno parameter?")}

      test_tbl <- test_tbl %>%
        spread(side, mutant) %>%
        select(LHS, RHS, genotype, conditions, -count) %>%
        mutate(raw_data = pmap(list(.$LHS, .$RHS, .$genotype, .$conditions),
                               ~ filter(data, mutant == ..1 | mutant == ..2,
                                        genotype == ..3,
                                        conditions == ..4)),
               sowing_numbers = map(raw_data, ~ unique(.x$sowing_number)))
    }


  }
  else if(quo_name(group) == "sowing_number"){

    test_tbl <- test_tbl %>%
      mutate(raw_data = map2(LHS, RHS, ~ filter(data, .x == sowing_number | .y == sowing_number)))
  }

  # Select relevant data
  if(species == "Medicago"){
    test_tbl <- test_tbl %>%
      mutate(raw_data = map(raw_data, ~select(.x, !!group, days_to_flower, node_num)))
  }
  else if(species == "Arabidopsis"){
    test_tbl <- test_tbl %>%
      mutate(raw_data = map(raw_data, ~ select(.x, !!group, days_to_flower, total_leaves)))
  }

  # corr, means, sd
  test_tbl <- test_tbl %>%
    mutate(cor = map_dbl(raw_data, ~ cor(.x[,2], .x[,3], use = "na.or.complete")),
           mean = pmap(list(raw_data, .$LHS, .$RHS), ~ .getSumCols(..1, func = mean, !!group, ..2, ..3)),
           mean = map(mean, ~ set_colnames(.x, colnames(.x) %>%
                                             str_replace("..2", "LHS") %>%
                                             str_replace("..3", "RHS"))),

           sd = pmap(list(raw_data, .$LHS, .$RHS), ~ .getSumCols(..1, func = sd, !!group, ..2, ..3)),
           sd = map(sd, ~ set_colnames(.x, colnames(.x) %>%
                                         str_replace("..2", "LHS") %>%
                                         str_replace("..3", "RHS"))),

           # Testing
           raw_data = map(raw_data, ~ split(.x, .x[,1])),
           test = suppressMessages(map(raw_data, ~ ICSNP::HotellingsT2(.x[[1]][,2:3], .x[[2]][,2:3], na.action = na.exclude) %>%
                                         broom::tidy(quietly()) %>%
                                         select(T2_statistic = statistic,
                                                df1 = df1,
                                                df2 = df2,
                                                p_value = p.value)))

    )

  # # Switch raw_data back
  test_tbl <- test_tbl %>%
    mutate(raw_data = map(raw_data, ~ bind_rows(.x)))

  # Unnest data_frames
  test_tbl <- test_tbl %>%
    unnest(mean) %>%
    unnest(sd) %>%
    unnest(test)

  if(quo_name(group) == "genotype"){
    test_tbl <- test_tbl %>%
      mutate_at(vars(6:15), funs(round(., 2)))
  }
  else if(quo_name(group) == "mutant"){
    test_tbl <- test_tbl %>%
      mutate_at(vars(7:16), funs(round(., 2)))}
  else{
    test_tbl <- test_tbl %>%
      mutate_at(vars(4:13), funs(round(., 2)))
  }

  # Adjust p-values
  if(nrow(test_tbl) > 1){
    test_tbl <- test_tbl %>%
      mutate(p_adjust = p.adjust(p_value, method = "fdr"))
  }

  return(test_tbl)
  }
