#' list_to_df
#'
#' @param data a dataframe
#' @param func a summary function
#' @param col  the column on which to split
#' @param ... the groups within col to select if you only want a subset
#' @description Summarises a given column grouped on a second using a
#' summary function and produces a wide dataframe with column names
#' reflecting the group and summary function
#' @examples
#' .getSumCols(iris[,c(1,5)], mean, Species)
#'
#' AA <- "setosa"
#' BB <- "versicolor"
#' .getSumCols(iris[,c(1,5)], mean, Species, AA, BB)
.getSumCols <- function(data, func, col, ...){

  require(rlang)
  require(dplyr)
  require(tidyr)
  require(purrr)
  require(magrittr)

  enq_col <- enquo(col)
  enq_vars <- enquos(...)

  func_name <- as.character(substitute(func))

  if(length(enq_vars) > 0) {
    data <- filter(data, !! enq_col %in% map_chr(enq_vars, ~ eval_tidy(.x)))
  }

  data <- data %>%
    group_by(!! enq_col) %>%
    summarise_if(is.numeric, func, na.rm = TRUE) %>%
    gather(measurement, value, -!! enq_col) %>%
    mutate_if(is.factor, as.character)

  if(length(enq_vars) > 0) {
    lookup_table <- data_frame(vals = unname(map_chr(enq_vars, ~ eval_tidy(.x))),
                               names = map_chr(enq_vars, ~ quo_name(.x))) %>%
      set_colnames(c(quo_name(enq_col), "names"))

    data <- data %>%
      left_join(lookup_table, by = quo_name(enq_col))
  }
  else{
    data <- data %>%
      mutate(names = !! enq_col)
  }

  data %>%
    mutate(measurement = paste(func_name, measurement, sep = "_")) %>%
    unite(var, names, measurement) %>%
    select(var, value) %>%
    spread(var, value)
}
