#' rt_test
#'
#' @param data a dataframe from `rt_summary()` with the following columns - "date", "target",
#' "sample", "genotype", "condition", "bio_replicate", "mean_ct", "sd_div_mean", "sd_check"
#' @param reference The name of your housekeeper gene (default set to MtPP2A)
#' @description Performs the ddCt method on a dataframe from `rt_summary()`.
#'
#' @export
rt_test <- function(data, reference = MtPP2A){

  require(dplyr)
  require(tidyr)
  require(stringr)
  require(purrr)
  require(magrittr)

  # Make reference a quosure
  reference <- enquo(reference)

  # Check data format
  assertNames(names(data), identical.to = c("date", "target", "sample", "genotype",
                                            "condition", "bio_replicate", "mean_ct",
                                            "sd_div_mean", "sd_check"),
              .var.name = "data")

  # Check the reference gene is there
  if(!quo_name(reference) %in% (data %>% pull(target))){

    stop("ERROR: Your reference gene isn't in the dataset.")
  }

  # Perform the ddCt method
  data %>%
    ungroup() %>%
    select(-sd_div_mean, -sd_check) %>%
    spread(target, mean_ct) %>%
    mutate(sample = as.character(sample)) %>% # for future sanity
    mutate_at(vars(-one_of(c("date","sample", "genotype", "condition", "bio_replicate", quo_name(reference)))),
              funs(dCt = . - !!reference)) %>%
    mutate_at(vars(contains("dCt")),
              funs(ddCt = . - min(.,  na.rm = TRUE))) %>%
    mutate_at(vars(contains("ddCt")),
              funs(comp_exp = 2 ^ - .)) %>%
    filter(!is.na(ddCt)) %>% # problem with multiple genes at once.
    nest(-sample, -genotype, -condition) %>%



    mutate(mean = map(data, ~ summarise_at(.x, vars(contains("comp_exp")),
                                           funs(mean_expr = mean(., na.rm = T) ))), # T NOT TRUE for some reason
           se   = map(data, ~ summarise_at(.x, vars(contains("comp_exp")),
                                           funs(se = sd(., na.rm = T)/sqrt(sum(!is.na(.))) ))), # T NOT TRUE for some reason

           # Shorten names
           data = map(data, ~ set_colnames(.x, str_replace(names(.), "_dCt_", "_"))),
           data = map(data, ~ set_colnames(.x, str_replace(names(.), "_ddCt_", "_"))),
           mean = map(mean, ~ set_colnames(.x, str_replace(names(.), "_dCt_ddCt_comp_exp_", "_"))),
           se = map(se, ~ set_colnames(.x, str_replace(names(.), "_dCt_ddCt_comp_exp_", "_")))
    ) %>%
    unnest(mean, se) %>%
    select(1:4, order(colnames(.)))
}
